Files
TheRock/rocm-libraries/projects/rocsolver/clients/common/lapack/testing_syev_heev.hpp
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

685 lines
27 KiB
C++

/* **************************************************************************
* Copyright (C) 2021-2026 Advanced Micro Devices, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* 1. Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGE.
* *************************************************************************/
#pragma once
#include "common/misc/client_util.hpp"
#include "common/misc/clientcommon.hpp"
#include "common/misc/lapack_host_reference.hpp"
#include "common/misc/norm.hpp"
#include "common/misc/rocsolver.hpp"
#include "common/misc/rocsolver_arguments.hpp"
#include "common/misc/rocsolver_test.hpp"
#include "common/misc/rocsolver_timer.hpp"
template <bool STRIDED, typename I, typename T, typename S, typename U>
void syev_heev_checkBadArgs(const rocblas_handle handle,
const rocblas_evect evect,
const rocblas_fill uplo,
const I n,
T dA,
const I lda,
const rocblas_stride stA,
S dD,
const rocblas_stride stD,
S dE,
const rocblas_stride stE,
U dinfo,
const I bc)
{
// handle
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, nullptr, evect, uplo, n, dA, lda, stA, dD,
stD, dE, stE, dinfo, bc),
rocblas_status_invalid_handle);
// values
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, rocblas_evect(0), uplo, n, dA, lda,
stA, dD, stD, dE, stE, dinfo, bc),
rocblas_status_invalid_value);
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, rocblas_fill_full, n, dA, lda,
stA, dD, stD, dE, stE, dinfo, bc),
rocblas_status_invalid_value);
// sizes (only check batch_count if applicable)
if(STRIDED)
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA, lda, stA, dD,
stD, dE, stE, dinfo, I(-1)),
rocblas_status_invalid_size);
// pointers
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, (T) nullptr, lda,
stA, dD, stD, dE, stE, dinfo, bc),
rocblas_status_invalid_pointer);
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA, lda, stA,
(S) nullptr, stD, dE, stE, dinfo, bc),
rocblas_status_invalid_pointer);
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA, lda, stA, dD,
stD, (S) nullptr, stE, dinfo, bc),
rocblas_status_invalid_pointer);
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA, lda, stA, dD,
stD, dE, stE, (U) nullptr, bc),
rocblas_status_invalid_pointer);
// quick return with invalid pointers
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, I(0), (T) nullptr, lda,
stA, (S) nullptr, stD, (S) nullptr, stE, dinfo, bc),
rocblas_status_success);
// quick return with zero batch_count if applicable
if(STRIDED)
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA, lda, stA, dD,
stD, dE, stE, (U) nullptr, I(0)),
rocblas_status_success);
}
template <bool BATCHED, bool STRIDED, typename T, typename I>
void testing_syev_heev_bad_arg()
{
using S = decltype(std::real(T{}));
// safe arguments
rocblas_local_handle handle;
rocblas_evect evect = rocblas_evect_none;
rocblas_fill uplo = rocblas_fill_lower;
I n = 1;
I lda = 1;
rocblas_stride stA = 1;
rocblas_stride stD = 1;
rocblas_stride stE = 1;
I bc = 1;
#ifndef ROCSOLVER_ENABLE_EIGENSOLVERS_64
// 64-bit API disabled: entry points must report not_implemented (see note in testing_syev_heev).
if constexpr(std::is_same<I, int64_t>::value)
{
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, (T*)nullptr, lda,
stA, (S*)nullptr, stD, (S*)nullptr, stE,
(I*)nullptr, bc),
rocblas_status_not_implemented);
return;
}
#endif
if(BATCHED)
{
// memory allocations
device_batch_vector<T> dA(1, 1, 1);
device_strided_batch_vector<S> dD(1, 1, 1, 1);
device_strided_batch_vector<S> dE(1, 1, 1, 1);
device_strided_batch_vector<I> dinfo(1, 1, 1, 1);
CHECK_HIP_ERROR(dA.memcheck());
CHECK_HIP_ERROR(dD.memcheck());
CHECK_HIP_ERROR(dE.memcheck());
CHECK_HIP_ERROR(dinfo.memcheck());
// check bad arguments
syev_heev_checkBadArgs<STRIDED, I>(handle, evect, uplo, n, dA.data(), lda, stA, dD.data(),
stD, dE.data(), stE, dinfo.data(), bc);
}
else
{
// memory allocations
device_strided_batch_vector<T> dA(1, 1, 1, 1);
device_strided_batch_vector<S> dD(1, 1, 1, 1);
device_strided_batch_vector<S> dE(1, 1, 1, 1);
device_strided_batch_vector<I> dinfo(1, 1, 1, 1);
CHECK_HIP_ERROR(dA.memcheck());
CHECK_HIP_ERROR(dD.memcheck());
CHECK_HIP_ERROR(dE.memcheck());
CHECK_HIP_ERROR(dinfo.memcheck());
// check bad arguments
syev_heev_checkBadArgs<STRIDED, I>(handle, evect, uplo, n, dA.data(), lda, stA, dD.data(),
stD, dE.data(), stE, dinfo.data(), bc);
}
}
template <bool CPU, bool GPU, typename T, typename I, typename Td, typename Th>
void syev_heev_initData(const rocblas_handle handle,
const rocblas_evect evect,
const I n,
Td& dA,
const I lda,
const I bc,
Th& hA,
std::vector<T>& A,
bool test = true)
{
if(CPU)
{
rocblas_init<T>(hA, true);
// scale A to avoid singularities
for(I b = 0; b < bc; ++b)
{
for(I i = 0; i < n; i++)
{
for(I j = 0; j < n; j++)
{
if(i == j)
hA[b][i + j * lda] = std::real(hA[b][i + j * lda]) + 400;
else
hA[b][i + j * lda] -= 4;
}
}
// make copy of original data to test vectors if required
if(test && evect == rocblas_evect_original)
{
for(I i = 0; i < n; i++)
{
for(I j = 0; j < n; j++)
A[b * lda * n + i + j * lda] = hA[b][i + j * lda];
}
}
}
}
if(GPU)
{
// now copy to the GPU
CHECK_HIP_ERROR(dA.transfer_from(hA));
}
}
template <bool STRIDED, typename T, typename I, typename Sd, typename Td, typename Id, typename Sh, typename Th, typename Ih>
void syev_heev_getError(const rocblas_handle handle,
const rocblas_evect evect,
const rocblas_fill uplo,
const I n,
Td& dA,
const I lda,
const rocblas_stride stA,
Sd& dD,
const rocblas_stride stD,
Sd& dE,
const rocblas_stride stE,
Id& dinfo,
const I bc,
Th& hA,
Th& hAres,
Sh& hD,
Sh& hDres,
Ih& hinfo,
Ih& hinfoRes,
double* max_err)
{
constexpr bool COMPLEX = rocblas_is_complex<T>;
using S = decltype(std::real(T{}));
int sizeE = 3 * n - 1;
int lwork = (COMPLEX ? 2 * n - 1 : 0);
std::vector<T> work(lwork);
std::vector<S> hE(sizeE);
std::vector<T> A(lda * n * bc);
// input data initialization
syev_heev_initData<true, true, T>(handle, evect, n, dA, lda, bc, hA, A);
// execute computations
// GPU lapack
CHECK_ROCBLAS_ERROR(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA.data(), lda, stA,
dD.data(), stD, dE.data(), stE, dinfo.data(), bc));
CHECK_HIP_ERROR(hDres.transfer_from(dD));
CHECK_HIP_ERROR(hinfoRes.transfer_from(dinfo));
if(evect == rocblas_evect_original)
CHECK_HIP_ERROR(hAres.transfer_from(dA));
// CPU lapack
for(I b = 0; b < bc; ++b)
{
rocblas_int info_tmp = 0;
cpu_syev_heev(evect, uplo, n, hA[b], lda, hD[b], work.data(), lwork, hE.data(), sizeE,
&info_tmp);
hinfo[b][0] = info_tmp;
}
// Check info for non-convergence
*max_err = 0;
for(I b = 0; b < bc; ++b)
{
EXPECT_EQ(hinfo[b][0], hinfoRes[b][0]) << "where b = " << b;
if(hinfo[b][0] != hinfoRes[b][0])
*max_err += 1;
}
// (We expect the used input matrices to always converge. Testing
// implicitly the equivalent non-converged matrix is very complicated and it boils
// down to essentially run the algorithm again and until convergence is achieved).
double err = 0;
for(I b = 0; b < bc; ++b)
{
if(evect != rocblas_evect_original)
{
// only eigenvalues needed; can compare with LAPACK
// error is ||hD - hDRes|| / ||hD||
// using frobenius norm
if(hinfo[b][0] == 0)
err = norm_error('F', 1, n, 1, hD[b], hDres[b]);
*max_err = err > *max_err ? err : *max_err;
}
else
{
// both eigenvalues and eigenvectors needed; need to implicitly test
// eigenvectors due to non-uniqueness of eigenvectors under scaling
if(hinfo[b][0] == 0)
{
// multiply A with each of the n eigenvectors and divide by corresponding
// eigenvalues
T alpha;
T beta = 0;
for(I j = 0; j < n; j++)
{
alpha = T(1) / hDres[b][j];
cpu_symv_hemv(uplo, n, alpha, A.data() + b * lda * n, lda, hAres[b] + j * lda,
1, beta, hA[b] + j * lda, 1);
}
// error is ||hA - hARes|| / ||hA||
// using frobenius norm
err = norm_error('F', n, n, lda, hA[b], hAres[b]);
*max_err = err > *max_err ? err : *max_err;
}
}
}
}
template <bool STRIDED, typename T, typename I, typename Sd, typename Td, typename Id, typename Sh, typename Th, typename Ih>
void syev_heev_getPerfData(const rocblas_handle handle,
const rocblas_evect evect,
const rocblas_fill uplo,
const I n,
Td& dA,
const I lda,
const rocblas_stride stA,
Sd& dD,
const rocblas_stride stD,
Sd& dE,
const rocblas_stride stE,
Id& dinfo,
const I bc,
Th& hA,
Sh& hD,
Ih& hinfo,
double* gpu_time_used,
double* cpu_time_used,
const rocblas_int hot_calls,
const int profile,
const bool profile_kernels,
const bool perf)
{
constexpr bool COMPLEX = rocblas_is_complex<T>;
using S = decltype(std::real(T{}));
int sizeE = 3 * n - 1;
int lwork = (COMPLEX ? 2 * n - 1 : 0);
std::vector<T> work(lwork);
std::vector<S> hE(sizeE);
std::vector<T> A;
if(!perf)
{
syev_heev_initData<true, false, T>(handle, evect, n, dA, lda, bc, hA, A, 0);
// cpu-lapack performance (only if not in perf mode)
*cpu_time_used = get_time_us_no_sync();
for(I b = 0; b < bc; ++b)
{
rocblas_int info_tmp = 0;
cpu_syev_heev(evect, uplo, n, hA[b], lda, hD[b], work.data(), lwork, hE.data(), sizeE,
&info_tmp);
hinfo[b][0] = info_tmp;
}
*cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
}
syev_heev_initData<true, false, T>(handle, evect, n, dA, lda, bc, hA, A, 0);
// cold calls
for(int iter = 0; iter < 2; iter++)
{
syev_heev_initData<false, true, T>(handle, evect, n, dA, lda, bc, hA, A, 0);
CHECK_ROCBLAS_ERROR(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA.data(), lda, stA,
dD.data(), stD, dE.data(), stE, dinfo.data(), bc));
}
// gpu-lapack performance
hipStream_t stream;
CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
rocsolver_timer timer;
if(profile > 0)
{
if(profile_kernels)
rocsolver_log_set_layer_mode(rocblas_layer_mode_log_profile
| rocblas_layer_mode_ex_log_kernel);
else
rocsolver_log_set_layer_mode(rocblas_layer_mode_log_profile);
rocsolver_log_set_max_levels(profile);
}
for(rocblas_int iter = 0; iter < hot_calls; iter++)
{
syev_heev_initData<false, true, T>(handle, evect, n, dA, lda, bc, hA, A, 0);
timer.start(stream);
rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA.data(), lda, stA, dD.data(), stD,
dE.data(), stE, dinfo.data(), bc);
timer.end(stream);
}
*gpu_time_used = timer.get_combined();
}
template <bool BATCHED, bool STRIDED, typename T, typename I>
void testing_syev_heev(Arguments& argus)
{
using S = decltype(std::real(T{}));
// get arguments
rocblas_local_handle handle;
char evectC = argus.get<char>("evect");
char uploC = argus.get<char>("uplo");
I n = argus.get<rocblas_int>("n");
I lda = argus.get<rocblas_int>("lda", n);
rocblas_stride stA = argus.get<rocblas_stride>("strideA", lda * n);
rocblas_stride stD = argus.get<rocblas_stride>("strideD", n);
rocblas_stride stE = argus.get<rocblas_stride>("strideE", n);
rocblas_evect evect = char2rocblas_evect(evectC);
rocblas_fill uplo = char2rocblas_fill(uploC);
I bc = argus.batch_count;
rocblas_int hot_calls = argus.iters;
#ifndef ROCSOLVER_ENABLE_EIGENSOLVERS_64
// The 64-bit eigensolver API is gated behind the ROCSOLVER_ENABLE_EIGENSOLVERS_64
// feature flag; when the flag is off the _64 entry points must report
// rocblas_status_not_implemented instead of running.
if constexpr(std::is_same<I, int64_t>::value)
{
if(BATCHED)
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n,
(T* const*)nullptr, lda, stA, (S*)nullptr,
stD, (S*)nullptr, stE, (I*)nullptr, bc),
rocblas_status_not_implemented);
else
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, (T*)nullptr,
lda, stA, (S*)nullptr, stD, (S*)nullptr, stE,
(I*)nullptr, bc),
rocblas_status_not_implemented);
return;
}
#endif
if(argus.alg_mode == 1)
{
EXPECT_ROCBLAS_STATUS(
rocsolver_set_alg_mode(handle, rocsolver_function_syev_heev, rocsolver_alg_mode_hybrid),
rocblas_status_success);
rocsolver_alg_mode sterf_mode, steqr_mode;
EXPECT_ROCBLAS_STATUS(rocsolver_get_alg_mode(handle, rocsolver_function_sterf, &sterf_mode),
rocblas_status_success);
EXPECT_ROCBLAS_STATUS(rocsolver_get_alg_mode(handle, rocsolver_function_steqr, &steqr_mode),
rocblas_status_success);
EXPECT_EQ(sterf_mode, rocsolver_alg_mode_hybrid);
EXPECT_EQ(steqr_mode, rocsolver_alg_mode_hybrid);
}
// check non-supported values
if(uplo == rocblas_fill_full || evect == rocblas_evect_tridiagonal)
{
if(BATCHED)
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n,
(T* const*)nullptr, lda, stA, (S*)nullptr,
stD, (S*)nullptr, stE, (I*)nullptr, bc),
rocblas_status_invalid_value);
else
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, (T*)nullptr,
lda, stA, (S*)nullptr, stD, (S*)nullptr, stE,
(I*)nullptr, bc),
rocblas_status_invalid_value);
if(argus.timing)
rocsolver_bench_inform(inform_invalid_args);
return;
}
// determine sizes
size_t size_A = size_t(lda) * n;
size_t size_D = n;
size_t size_E = size_D;
size_t size_Ares = (argus.unit_check || argus.norm_check) ? size_A : 0;
size_t size_Dres = (argus.unit_check || argus.norm_check) ? size_D : 0;
double max_error = 0, gpu_time_used = 0, cpu_time_used = 0;
// check invalid sizes
bool invalid_size = (n < 0 || lda < n || bc < 0);
if(invalid_size)
{
if(BATCHED)
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n,
(T* const*)nullptr, lda, stA, (S*)nullptr,
stD, (S*)nullptr, stE, (I*)nullptr, bc),
rocblas_status_invalid_size);
else
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, (T*)nullptr,
lda, stA, (S*)nullptr, stD, (S*)nullptr, stE,
(I*)nullptr, bc),
rocblas_status_invalid_size);
if(argus.timing)
rocsolver_bench_inform(inform_invalid_size);
return;
}
// memory size query is necessary
if(argus.mem_query)
{
CHECK_ROCBLAS_ERROR(rocblas_start_device_memory_size_query(handle));
if(BATCHED)
CHECK_ALLOC_QUERY(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n,
(T* const*)nullptr, lda, stA, (S*)nullptr, stD,
(S*)nullptr, stE, (I*)nullptr, bc));
else
CHECK_ALLOC_QUERY(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, (T*)nullptr, lda,
stA, (S*)nullptr, stD, (S*)nullptr, stE,
(I*)nullptr, bc));
size_t size;
CHECK_ROCBLAS_ERROR(rocblas_stop_device_memory_size_query(handle, &size));
rocsolver_bench_inform(inform_mem_query, size);
return;
}
// memory allocations (all cases)
// host
host_strided_batch_vector<S> hD(size_D, 1, stD, bc);
host_strided_batch_vector<I> hinfo(1, 1, 1, bc);
host_strided_batch_vector<I> hinfoRes(1, 1, 1, bc);
host_strided_batch_vector<S> hDres(size_Dres, 1, stD, bc);
// device
device_strided_batch_vector<S> dE(size_E, 1, stE, bc);
device_strided_batch_vector<S> dD(size_D, 1, stD, bc);
device_strided_batch_vector<I> dinfo(1, 1, 1, bc);
if(size_E)
CHECK_HIP_ERROR(dE.memcheck());
if(size_D)
CHECK_HIP_ERROR(dD.memcheck());
CHECK_HIP_ERROR(dinfo.memcheck());
if(BATCHED)
{
// memory allocations
host_batch_vector<T> hA(size_A, 1, bc);
host_batch_vector<T> hAres(size_Ares, 1, bc);
device_batch_vector<T> dA(size_A, 1, bc);
if(size_A)
CHECK_HIP_ERROR(dA.memcheck());
// check quick return
if(n == 0 || bc == 0)
{
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA.data(),
lda, stA, dD.data(), stD, dE.data(), stE,
dinfo.data(), bc),
rocblas_status_success);
if(argus.timing)
rocsolver_bench_inform(inform_quick_return);
return;
}
// check computations
if(argus.unit_check || argus.norm_check)
{
syev_heev_getError<STRIDED, T, I>(handle, evect, uplo, n, dA, lda, stA, dD, stD, dE,
stE, dinfo, bc, hA, hAres, hD, hDres, hinfo, hinfoRes,
&max_error);
}
// collect performance data
if(argus.timing && hot_calls > 0)
{
syev_heev_getPerfData<STRIDED, T, I>(handle, evect, uplo, n, dA, lda, stA, dD, stD, dE,
stE, dinfo, bc, hA, hD, hinfo, &gpu_time_used,
&cpu_time_used, hot_calls, argus.profile,
argus.profile_kernels, argus.perf);
}
}
else
{
// memory allocations
host_strided_batch_vector<T> hA(size_A, 1, stA, bc);
host_strided_batch_vector<T> hAres(size_Ares, 1, stA, bc);
device_strided_batch_vector<T> dA(size_A, 1, stA, bc);
if(size_A)
CHECK_HIP_ERROR(dA.memcheck());
// check quick return
if(n == 0 || bc == 0)
{
EXPECT_ROCBLAS_STATUS(rocsolver_syev_heev(STRIDED, handle, evect, uplo, n, dA.data(),
lda, stA, dD.data(), stD, dE.data(), stE,
dinfo.data(), bc),
rocblas_status_success);
if(argus.timing)
rocsolver_bench_inform(inform_quick_return);
return;
}
// check computations
if(argus.unit_check || argus.norm_check)
{
syev_heev_getError<STRIDED, T, I>(handle, evect, uplo, n, dA, lda, stA, dD, stD, dE,
stE, dinfo, bc, hA, hAres, hD, hDres, hinfo, hinfoRes,
&max_error);
}
// collect performance data
if(argus.timing && hot_calls > 0)
{
syev_heev_getPerfData<STRIDED, T, I>(handle, evect, uplo, n, dA, lda, stA, dD, stD, dE,
stE, dinfo, bc, hA, hD, hinfo, &gpu_time_used,
&cpu_time_used, hot_calls, argus.profile,
argus.profile_kernels, argus.perf);
}
}
// validate results for rocsolver-test
// using n * machine_precision as tolerance
if(argus.unit_check)
ROCSOLVER_TEST_CHECK(T, max_error, n);
// output results for rocsolver-bench
if(argus.timing)
{
if(!argus.perf)
{
rocsolver_bench_header("Arguments:");
if(BATCHED)
{
rocsolver_bench_output("evect", "uplo", "n", "lda", "strideD", "strideE", "batch_c");
rocsolver_bench_output(evectC, uploC, n, lda, stD, stE, bc);
}
else if(STRIDED)
{
rocsolver_bench_output("evect", "uplo", "n", "lda", "strideA", "strideD", "strideE",
"batch_c");
rocsolver_bench_output(evectC, uploC, n, lda, stA, stD, stE, bc);
}
else
{
rocsolver_bench_output("evect", "uplo", "n", "lda");
rocsolver_bench_output(evectC, uploC, n, lda);
}
rocsolver_bench_header("Results:");
if(argus.norm_check)
{
rocsolver_bench_output("cpu_time_us", "gpu_time_us", "error");
rocsolver_bench_output(cpu_time_used, gpu_time_used, max_error);
}
else
{
rocsolver_bench_output("cpu_time_us", "gpu_time_us");
rocsolver_bench_output(cpu_time_used, gpu_time_used);
}
rocsolver_bench_endl();
}
else
{
if(argus.norm_check)
rocsolver_bench_output(gpu_time_used, max_error);
else
rocsolver_bench_output(gpu_time_used);
}
}
// ensure all arguments were consumed
argus.validate_consumed();
}
#define EXTERN_TESTING_SYEV_HEEV(...) \
extern template void testing_syev_heev<__VA_ARGS__>(Arguments&);
INSTANTIATE(EXTERN_TESTING_SYEV_HEEV,
FOREACH_MATRIX_DATA_LAYOUT,
FOREACH_SCALAR_TYPE,
FOREACH_INT_TYPE,
APPLY_STAMP)